'use strict';
const bodyParser = require('body-parser');
const debug = require('debug')('benchmark:express');
const express = require('express');
const fs = require('fs');
const headerToken = require('../../source/server/header-token');
const httpCodes = require('http-codes');
const helmet = require('helmet');
const prettyHrtime = require('pretty-hrtime');
const toobusy = require('toobusy-js');
const yaml = require('js-yaml');

const start = process.hrtime();

const configuration = require('../configuration')
  .server;
const resources = require('../data');
const apiDoc = yaml.safeLoad(fs.readFileSync('../centro/v1/api.yaml'), 'utf8');

debug('start');

const app = express();
addGeneralMiddleware(app);
configure(app);
const server = new createServer(app, configuration);

app.get('/v1/resources/:id', (request, response) => {
  response.send(resources);
});

app.get('/api-documentation/v1', (request, response) => {
  response.send(apiDoc);
});

const port = server.start();
const startupTime = process.hrtime(start);
debug('server:start', 'success', 'PORT', port, 'time to start', prettyHrtime(startupTime));

function addGeneralMiddleware(app) {
  app.use((request, response, next) => {
    if (toobusy()) response.sendStatus(httpCodes.SERVICE_UNAVAILABLE);
    else next();
  });
  app.use(bodyParser.json());
  app.use(helmet());
  app.use(headerToken);
}

function configure(app) {
  app.disable('x-powered-by');
}

function createServer(app, configuration) {
  this.running = false;

  const gracefulShutdown = () => {
    if (this.running) {
      debug('stop:attempt');
      this.server.close(() => {
        debug('stop:success');
        this.running = false;
        process.exit();
      });
      setTimeout(function() {
        debug('stop:forced', 'could not close connections in time, forcefully shutting down');
        this.running = false;
        process.exit();
      }, configuration.timeout);
    }
  };

  const start = () => {
    debug('start:attempt');
    this.server = app.listen(process.env.PORT || configuration.port);
    this.running = true;
    debug('start:success', 'PORT', this.server.address()
      .port);
    process.on('SIGTERM', gracefulShutdown);
    process.on('SIGINT', gracefulShutdown);
    return this.server.address()
      .port;
  };

  return {
    start: start,
    shutdown: gracefulShutdown,
  };
}
